/*
 * Copyright (C) 2008-2012 TrinityCore <http://www.trinitycore.org/>
 * Copyright (C) 2005-2009 MaNGOS <http://getmangos.com/>
 *
 * This program is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the
 * Free Software Foundation; either version 2 of the License, or (at your
 * option) any later version.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for
 * more details.
 *
 * You should have received a copy of the GNU General Public License along
 * with this program. If not, see <http://www.gnu.org/licenses/>.
 */
#ifndef TRINITY_SPELLAURADEFINES_H
#define TRINITY_SPELLAURADEFINES_H

#define MAX_AURAS 255

// 5.0.5
enum AURA_FLAGS
{
    AFLAG_NONE                   = 0x00,
    AFLAG_NOCASTER               = 0x01,
    AFLAG_POSITIVE               = 0x02,
    AFLAG_DURATION               = 0x04,
    AFLAG_SCALABLE               = 0x08,
    AFLAG_NEGATIVE               = 0x10,
    AFLAG_FAKEAURA               = 0x20, //May be fake auras or not visible or update fake aura?
};

// these are modes, in which aura effect handler may be called

enum AuraEffectHandleModes
{
    AURA_EFFECT_HANDLE_DEFAULT = 0x0,
    AURA_EFFECT_HANDLE_REAL = 0x01, // handler applies/removes effect from unit
    AURA_EFFECT_HANDLE_SEND_FOR_CLIENT = 0x02, // handler sends apply/remove packet to unit
    AURA_EFFECT_HANDLE_CHANGE_AMOUNT = 0x04, // handler updates effect on target after effect amount change
    AURA_EFFECT_HANDLE_REAPPLY = 0x08, // handler updates effect on target after aura is reapplied on target
    AURA_EFFECT_HANDLE_STAT = 0x10, // handler updates effect on target when stat removal/apply is needed for calculations by core
    AURA_EFFECT_HANDLE_SKILL = 0x20, // handler updates effect on target when skill removal/apply is needed for calculations by core
    AURA_EFFECT_HANDLE_SEND_FOR_CLIENT_MASK = (AURA_EFFECT_HANDLE_SEND_FOR_CLIENT | AURA_EFFECT_HANDLE_REAL), // any case handler need to send packet
    AURA_EFFECT_HANDLE_CHANGE_AMOUNT_MASK = (AURA_EFFECT_HANDLE_CHANGE_AMOUNT | AURA_EFFECT_HANDLE_REAL), // any case handler applies effect depending on amount
    AURA_EFFECT_HANDLE_CHANGE_AMOUNT_SEND_FOR_CLIENT_MASK = (AURA_EFFECT_HANDLE_CHANGE_AMOUNT_MASK | AURA_EFFECT_HANDLE_SEND_FOR_CLIENT_MASK),
    AURA_EFFECT_HANDLE_REAL_OR_REAPPLY_MASK = (AURA_EFFECT_HANDLE_REAPPLY | AURA_EFFECT_HANDLE_REAL),
};

//m_schoolAbsorb
enum DAMAGE_ABSORB_TYPE
{
    ALL_DAMAGE_ABSORB       = -2,
    ONLY_MAGIC_ABSORB       = -1,
};

enum AuraType
{
    SPELL_AURA_NONE                                         = 0,
    SPELL_AURA_BIND_SIGHT                                   = 1,
    SPELL_AURA_MOD_POSSESS                                  = 2,
    SPELL_AURA_PERIODIC_DAMAGE                              = 3,
    SPELL_AURA_DUMMY                                        = 4,
    SPELL_AURA_MOD_CONFUSE                                  = 5,
    SPELL_AURA_MOD_CHARM                                    = 6,
    SPELL_AURA_MOD_FEAR                                     = 7,
    SPELL_AURA_PERIODIC_HEAL                                = 8,
    SPELL_AURA_MOD_ATTACKSPEED                              = 9,
    SPELL_AURA_MOD_THREAT                                   = 10,
    SPELL_AURA_MOD_TAUNT                                    = 11,
    SPELL_AURA_MOD_STUN                                     = 12,
    SPELL_AURA_MOD_DAMAGE_DONE                              = 13,
    SPELL_AURA_MOD_DAMAGE_TAKEN                             = 14,
    SPELL_AURA_DAMAGE_SHIELD                                = 15,
    SPELL_AURA_MOD_STEALTH                                  = 16,
    SPELL_AURA_MOD_STEALTH_DETECT                           = 17,
    SPELL_AURA_MOD_INVISIBILITY                             = 18,
    SPELL_AURA_MOD_INVISIBILITY_DETECT                      = 19,
    SPELL_AURA_OBS_MOD_HEALTH                               = 20,  // 20, 21 unofficial
    SPELL_AURA_OBS_MOD_POWER                                = 21,
    SPELL_AURA_MOD_RESISTANCE                               = 22,
    SPELL_AURA_PERIODIC_TRIGGER_SPELL                       = 23,
    SPELL_AURA_PERIODIC_ENERGIZE                            = 24,
    SPELL_AURA_MOD_PACIFY                                   = 25,
    SPELL_AURA_MOD_ROOT                                     = 26,
    SPELL_AURA_MOD_SILENCE                                  = 27,
    SPELL_AURA_REFLECT_SPELLS                               = 28,
    SPELL_AURA_MOD_STAT                                     = 29,
    SPELL_AURA_MOD_SKILL                                    = 30,
    SPELL_AURA_MOD_INCREASE_SPEED                           = 31,
    SPELL_AURA_MOD_INCREASE_MOUNTED_SPEED                   = 32,
    SPELL_AURA_MOD_DECREASE_SPEED                           = 33,
    SPELL_AURA_MOD_INCREASE_HEALTH                          = 34,
    SPELL_AURA_MOD_INCREASE_ENERGY                          = 35,
    SPELL_AURA_MOD_SHAPESHIFT                               = 36,
    SPELL_AURA_EFFECT_IMMUNITY                              = 37,
    SPELL_AURA_STATE_IMMUNITY                               = 38,
    SPELL_AURA_SCHOOL_IMMUNITY                              = 39,
    SPELL_AURA_DAMAGE_IMMUNITY                              = 40,
    SPELL_AURA_DISPEL_IMMUNITY                              = 41,
    SPELL_AURA_PROC_TRIGGER_SPELL                           = 42,
    SPELL_AURA_PROC_TRIGGER_DAMAGE                          = 43,
    SPELL_AURA_TRACK_CREATURES                              = 44,
    SPELL_AURA_TRACK_RESOURCES                              = 45,
    SPELL_AURA_46                                           = 46,
    SPELL_AURA_MOD_PARRY_PERCENT                            = 47,
    SPELL_AURA_PERIODIC_TRIGGER_SPELL_BY_CLIENT             = 48,  // periodic spell dest
    SPELL_AURA_MOD_DODGE_PERCENT                            = 49,
    SPELL_AURA_MOD_CRITICAL_HEALING_AMOUNT                  = 50,
    SPELL_AURA_MOD_BLOCK_PERCENT                            = 51,
    SPELL_AURA_MOD_WEAPON_CRIT_PERCENT                      = 52,
    SPELL_AURA_PERIODIC_LEECH                               = 53,
    SPELL_AURA_MOD_HIT_CHANCE                               = 54,
    SPELL_AURA_MOD_SPELL_HIT_CHANCE                         = 55,
    SPELL_AURA_TRANSFORM                                    = 56,
    SPELL_AURA_MOD_SPELL_CRIT_CHANCE                        = 57,
    SPELL_AURA_MOD_INCREASE_SWIM_SPEED                      = 58,
    SPELL_AURA_MOD_DAMAGE_DONE_CREATURE                     = 59,
    SPELL_AURA_MOD_PACIFY_SILENCE                           = 60,
    SPELL_AURA_MOD_SCALE                                    = 61,
    SPELL_AURA_PERIODIC_HEALTH_FUNNEL                       = 62,
    SPELL_AURA_MOD_ADDITIONAL_POWER_COST                    = 63,
    SPELL_AURA_PERIODIC_MANA_LEECH                          = 64,
    SPELL_AURA_MOD_CASTING_SPEED_NOT_STACK                  = 65,
    SPELL_AURA_FEIGN_DEATH                                  = 66,
    SPELL_AURA_MOD_DISARM                                   = 67,
    SPELL_AURA_MOD_STALKED                                  = 68,
    SPELL_AURA_SCHOOL_ABSORB                                = 69,
    SPELL_AURA_PERIODIC_WEAPON_PERCENT_DAMAGE               = 70,
    SPELL_AURA_71                                           = 71,
    SPELL_AURA_MOD_POWER_COST_SCHOOL_PCT                    = 72,
    SPELL_AURA_MOD_POWER_COST_SCHOOL                        = 73,
    SPELL_AURA_REFLECT_SPELLS_SCHOOL                        = 74,
    SPELL_AURA_MOD_LANGUAGE                                 = 75,
    SPELL_AURA_FAR_SIGHT                                    = 76,
    SPELL_AURA_MECHANIC_IMMUNITY                            = 77,
    SPELL_AURA_MOUNTED                                      = 78,
    SPELL_AURA_MOD_DAMAGE_PERCENT_DONE                      = 79,
    SPELL_AURA_MOD_PERCENT_STAT                             = 80,
    SPELL_AURA_SPLIT_DAMAGE_PCT                             = 81,
    SPELL_AURA_WATER_BREATHING                              = 82,
    SPELL_AURA_MOD_BASE_RESISTANCE                          = 83,
    SPELL_AURA_MOD_REGEN                                    = 84,
    SPELL_AURA_MOD_POWER_REGEN                              = 85,
    SPELL_AURA_CHANNEL_DEATH_ITEM                           = 86,
    SPELL_AURA_MOD_DAMAGE_PERCENT_TAKEN                     = 87,
    SPELL_AURA_MOD_HEALTH_REGEN_PERCENT                     = 88,
    SPELL_AURA_PERIODIC_DAMAGE_PERCENT                      = 89,
    SPELL_AURA_ENABLE_PVP_STAT_SCALING                      = 90,
    SPELL_AURA_MOD_DETECT_RANGE                             = 91,
    SPELL_AURA_PREVENTS_FLEEING                             = 92,
    SPELL_AURA_MOD_UNATTACKABLE                             = 93,
    SPELL_AURA_INTERRUPT_REGEN                              = 94,
    SPELL_AURA_GHOST                                        = 95,
    SPELL_AURA_SPELL_MAGNET                                 = 96,
    SPELL_AURA_MANA_SHIELD                                  = 97,
    SPELL_AURA_MOD_SKILL_TALENT                             = 98,
    SPELL_AURA_MOD_ATTACK_POWER                             = 99,
    SPELL_AURA_AURAS_VISIBLE                                = 100,
    SPELL_AURA_MOD_RESISTANCE_PCT                           = 101,
    SPELL_AURA_MOD_MELEE_ATTACK_POWER_VERSUS                = 102,
    SPELL_AURA_MOD_TOTAL_THREAT                             = 103,
    SPELL_AURA_WATER_WALK                                   = 104,
    SPELL_AURA_FEATHER_FALL                                 = 105,
    SPELL_AURA_HOVER                                        = 106,
    SPELL_AURA_ADD_FLAT_MODIFIER                            = 107,
    SPELL_AURA_ADD_PCT_MODIFIER                             = 108,
    SPELL_AURA_ADD_TARGET_TRIGGER                           = 109,
    SPELL_AURA_MOD_POWER_REGEN_PERCENT                      = 110,
    SPELL_AURA_INTERCEPT_MELEE_RANGED_ATTACKS               = 111,
    SPELL_AURA_OVERRIDE_CLASS_SCRIPTS                       = 112,
    SPELL_AURA_MOD_RANGED_DAMAGE_TAKEN                      = 113,
    SPELL_AURA_MOD_RANGED_DAMAGE_TAKEN_PCT                  = 114,
    SPELL_AURA_MOD_HEALING                                  = 115,
    SPELL_AURA_MOD_REGEN_DURING_COMBAT                      = 116,
    SPELL_AURA_MOD_MECHANIC_RESISTANCE                      = 117,
    SPELL_AURA_MOD_HEALING_PCT                              = 118,
    SPELL_AURA_PVP_TALENTS                                  = 119,
    SPELL_AURA_UNTRACKABLE                                  = 120,
    SPELL_AURA_EMPATHY                                      = 121,
    SPELL_AURA_MOD_OFFHAND_DAMAGE_PCT                       = 122,
    SPELL_AURA_MOD_TARGET_RESISTANCE                        = 123,
    SPELL_AURA_MOD_RANGED_ATTACK_POWER                      = 124,
    SPELL_AURA_MOD_MELEE_DAMAGE_TAKEN                       = 125,
    SPELL_AURA_MOD_MELEE_DAMAGE_TAKEN_PCT                   = 126,
    SPELL_AURA_RANGED_ATTACK_POWER_ATTACKER_BONUS           = 127,
    SPELL_AURA_FIXATE                                       = 128,
    SPELL_AURA_MOD_SPEED_ALWAYS                             = 129,
    SPELL_AURA_MOD_MOUNTED_SPEED_ALWAYS                     = 130,
    SPELL_AURA_MOD_RANGED_ATTACK_POWER_VERSUS               = 131,
    SPELL_AURA_MOD_INCREASE_ENERGY_PERCENT                  = 132,
    SPELL_AURA_MOD_INCREASE_HEALTH_PERCENT                  = 133,
    SPELL_AURA_MOD_MANA_REGEN_INTERRUPT                     = 134,
    SPELL_AURA_MOD_HEALING_DONE                             = 135,
    SPELL_AURA_MOD_HEALING_DONE_PERCENT                     = 136,
    SPELL_AURA_MOD_TOTAL_STAT_PERCENTAGE                    = 137,
    SPELL_AURA_MOD_MELEE_HASTE                              = 138,
    SPELL_AURA_FORCE_REACTION                               = 139,
    SPELL_AURA_MOD_RANGED_HASTE                             = 140,
    SPELL_AURA_141                                          = 141,  // PvP Stats - Spec Stat Template - All Specs
    SPELL_AURA_MOD_BASE_RESISTANCE_PCT                      = 142,
    SPELL_AURA_MOD_COOLDOWN_SPEED_RATE                      = 143,
    SPELL_AURA_SAFE_FALL                                    = 144,
    SPELL_AURA_MOD_MAX_HEALTH_PCT                           = 145,
    SPELL_AURA_ALLOW_TAME_PET_TYPE                          = 146,
    SPELL_AURA_MECHANIC_IMMUNITY_MASK                       = 147,
    SPELL_AURA_148                                          = 148,
    SPELL_AURA_REDUCE_PUSHBACK                              = 149,  // Reduce Pushback
    SPELL_AURA_MOD_SHIELD_BLOCKVALUE_PCT                    = 150,
    SPELL_AURA_TRACK_STEALTHED                              = 151,  // Track Stealthed
    SPELL_AURA_MOD_DETECTED_RANGE                           = 152,  // Mod Detected Range
    SPELL_AURA_MOD_AUTO_ATTACK_RANGE                        = 153,
    SPELL_AURA_MOD_STEALTH_LEVEL                            = 154,  // Stealth Level Modifier
    SPELL_AURA_MOD_WATER_BREATHING                          = 155,  // Mod Water Breathing
    SPELL_AURA_MOD_REPUTATION_GAIN                          = 156,  // Mod Reputation Gain
    SPELL_AURA_PET_DAMAGE_MULTI                             = 157,  // Mod Pet Damage
    SPELL_AURA_ALLOW_TALENT_SWAPPING                        = 158,
    SPELL_AURA_NO_PVP_CREDIT                                = 159,
    SPELL_AURA_160                                          = 160,
    SPELL_AURA_MOD_HEALTH_REGEN_IN_COMBAT                   = 161,
    SPELL_AURA_POWER_BURN                                   = 162,
    SPELL_AURA_MOD_CRIT_DAMAGE_BONUS                        = 163,
    SPELL_AURA_CANT_BREATH                                  = 164,
    SPELL_AURA_MELEE_ATTACK_POWER_ATTACKER_BONUS            = 165,
    SPELL_AURA_MOD_ATTACK_POWER_PCT                         = 166,
    SPELL_AURA_MOD_RANGED_ATTACK_POWER_PCT                  = 167,
    SPELL_AURA_MOD_DAMAGE_DONE_VERSUS                       = 168,
    SPELL_AURA_SET_FFA_PVP_STATE                            = 169,
    SPELL_AURA_DETECT_AMORE                                 = 170,
    SPELL_AURA_MOD_SPEED_NOT_STACK                          = 171,
    SPELL_AURA_MOD_MOUNTED_SPEED_NOT_STACK                  = 172,
    SPELL_AURA_MOD_COOLDOWN_PCT                             = 173,
    SPELL_AURA_MOD_SPELL_DAMAGE_OF_STAT_PERCENT             = 174,  // by defeult intelect, dependent from SPELL_AURA_MOD_SPELL_HEALING_OF_STAT_PERCENT
    SPELL_AURA_MOD_SPELL_HEALING_OF_STAT_PERCENT            = 175,
    SPELL_AURA_SPIRIT_OF_REDEMPTION                         = 176,
    SPELL_AURA_AOE_CHARM                                    = 177,
    SPELL_AURA_MOD_MAX_MANA                                 = 178,
    SPELL_AURA_MOD_ATTACKER_SPELL_CRIT_CHANCE               = 179,
    SPELL_AURA_MOD_FLAT_SPELL_DAMAGE_VERSUS                 = 180,
    SPELL_AURA_MOD_SHIPMENT_RESOURCE_COST                   = 181,
    SPELL_AURA_DISABLE_LEGENDARY_ITEMS_PASSIVE_EFFECTS      = 182,
    SPELL_AURA_MOD_CRITICAL_DAMAGE                          = 183,
    SPELL_AURA_MOD_ATTACKER_MELEE_HIT_CHANCE                = 184,
    SPELL_AURA_MOD_ATTACKER_RANGED_HIT_CHANCE               = 185,
    SPELL_AURA_MOD_ATTACKER_SPELL_HIT_CHANCE                = 186,
    SPELL_AURA_MOD_ATTACKER_MELEE_CRIT_CHANCE               = 187,
    SPELL_AURA_MOD_ATTACKER_RANGED_CRIT_CHANCE              = 188,
    SPELL_AURA_MOD_RATING                                   = 189,
    SPELL_AURA_MOD_FACTION_REPUTATION_GAIN                  = 190,
    SPELL_AURA_USE_NORMAL_MOVEMENT_SPEED                    = 191,
    SPELL_AURA_MOD_MELEE_RANGED_HASTE                       = 192,
    SPELL_AURA_MELEE_SLOW                                   = 193,
    SPELL_AURA_MOD_TARGET_ABSORB_SCHOOL                     = 194,
    SPELL_AURA_195                                          = 195,
    SPELL_AURA_MOD_COOLDOWN                                 = 196,
    SPELL_AURA_MOD_ATTACKER_SPELL_AND_WEAPON_CRIT_CHANCE    = 197,
    SPELL_AURA_MOD_DODGE_CHANCE_FROM_CRIT_CHANCE_BY_MASK    = 198,
    SPELL_AURA_199                                          = 199,
    SPELL_AURA_MOD_XP_PCT                                   = 200,
    SPELL_AURA_FLY                                          = 201,
    SPELL_AURA_202                                          = 202,
    SPELL_AURA_PREVENT_INTERRUPT                            = 203,
    SPELL_AURA_PREVENT_CORPSE_RELEASE                       = 204,
    SPELL_AURA_MOD_ABILITY_COOLDOWN                         = 205,
    SPELL_AURA_MOD_INCREASE_VEHICLE_FLIGHT_SPEED            = 206,
    SPELL_AURA_MOD_INCREASE_MOUNTED_FLIGHT_SPEED            = 207,
    SPELL_AURA_MOD_INCREASE_FLIGHT_SPEED                    = 208,
    SPELL_AURA_MOD_MOUNTED_FLIGHT_SPEED_ALWAYS              = 209,
    SPELL_AURA_MOD_VEHICLE_SPEED_ALWAYS                     = 210,
    SPELL_AURA_MOD_FLIGHT_SPEED_NOT_STACK                   = 211,
    SPELL_AURA_MOD_HONOR_POINTS_GAIN_BY_TYPE_MASK           = 212,
    SPELL_AURA_MOD_RAGE_FROM_DAMAGE_DEALT                   = 213,
    SPELL_AURA_214                                          = 214, // unk use in 20782
    SPELL_AURA_ARENA_PREPARATION                            = 215,
    SPELL_AURA_HASTE_SPELLS                                 = 216,
    SPELL_AURA_MOD_MELEE_HASTE_2                            = 217,
    SPELL_AURA_MOD_ITEM_STATS_AND_PROCK_BY_PCT_WITH_MASK    = 218,
    SPELL_AURA_219                                          = 219,
    SPELL_AURA_220                                          = 220,
    SPELL_AURA_MOD_DETAUNT                                  = 221,
    SPELL_AURA_222                                          = 222,
    SPELL_AURA_223                                          = 223,
    SPELL_AURA_ENABLE_EXTRA_TALENT                          = 224, //unused
    SPELL_AURA_MOD_VISIBILITY_RANGE                         = 225,
    SPELL_AURA_PERIODIC_DUMMY                               = 226,
    SPELL_AURA_PERIODIC_TRIGGER_SPELL_WITH_VALUE            = 227,
    SPELL_AURA_DETECT_STEALTH                               = 228,
    SPELL_AURA_MOD_AOE_DAMAGE_AVOIDANCE                     = 229,
    SPELL_AURA_MOD_INCREASE_HEALTH_2                        = 230,
    SPELL_AURA_PROC_TRIGGER_SPELL_WITH_VALUE                = 231,
    SPELL_AURA_MECHANIC_DURATION_MOD                        = 232,
    SPELL_AURA_CHANGE_MODEL_FOR_ALL_HUMANOIDS               = 233,  // client-side only
    SPELL_AURA_MECHANIC_DURATION_MOD_NOT_STACK              = 234,
    SPELL_AURA_235                                          = 235,
    SPELL_AURA_CONTROL_VEHICLE                              = 236,
    SPELL_AURA_237                                          = 237,
    SPELL_AURA_238                                          = 238,
    SPELL_AURA_MOD_SCALE_2                                  = 239,
    SPELL_AURA_MOD_EXPERTISE                                = 240,
    SPELL_AURA_FORCE_MOVE_FORWARD                           = 241,
    SPELL_AURA_242                                          = 242,
    SPELL_AURA_MOD_FACTION                                  = 243,
    SPELL_AURA_COMPREHEND_LANGUAGE                          = 244,
    SPELL_AURA_MOD_AURA_DURATION_BY_DISPEL                  = 245,
    SPELL_AURA_246                                          = 246,
    SPELL_AURA_CLONE_CASTER                                 = 247,
    SPELL_AURA_248                                          = 248,
    SPELL_AURA_MOD_DAMAGE_PERCENT_DONE_BY_TARGET_AURA_MECHANIC = 249,
    SPELL_AURA_MOD_INCREASE_HEALTH_OR_PCT                   = 250,
    SPELL_AURA_MOD_ENEMY_DODGE                              = 251,
    SPELL_AURA_MOD_SPEED_SLOW_ALL                           = 252,
    SPELL_AURA_MOD_BLOCK_CRIT_CHANCE                        = 253,
    SPELL_AURA_MOD_DISARM_OFFHAND                           = 254,
    SPELL_AURA_MOD_MECHANIC_DAMAGE_TAKEN_PERCENT            = 255,
    SPELL_AURA_256                                          = 256,
    SPELL_AURA_257                                          = 257,
    SPELL_AURA_OVERRIDE_SUMMONED_OBJECT                     = 258,
    SPELL_AURA_MOD_HOT_PCT                                  = 259, // SPELL_AURA_MOD_HOT_PCT
    SPELL_AURA_SCREEN_EFFECT                                = 260,
    SPELL_AURA_PHASE                                        = 261,
    SPELL_AURA_ABILITY_IGNORE_AURASTATE                     = 262,
    SPELL_AURA_ALLOW_ONLY_ABILITY                           = 263,
    SPELL_AURA_DISABLE_AUTO_ATTACK                          = 264,
    SPELL_AURA_265                                          = 265, // unused
    SPELL_AURA_266                                          = 266, // unused
    SPELL_AURA_MOD_IMMUNE_AURA_APPLY_SCHOOL                 = 267,
    SPELL_AURA_268                                          = 268,
    SPELL_AURA_MOD_DAMAGE_TAKEN_FROM_CASTER                 = 269,
    SPELL_AURA_INCREASE_SCHOOL_DAMAGE_TAKEN                 = 270, // new name is SPELL_AURA_INCREASE_SCHOOL_DAMAGE_TAKEN - keep old behind recheck & update
    SPELL_AURA_MOD_DAMAGE_FROM_CASTER                       = 271,
    SPELL_AURA_272                                          = 272,
    SPELL_AURA_X_RAY                                        = 273,
    SPELL_AURA_274                                          = 274, // old SPELL_AURA_ABILITY_CONSUME_NO_AMMO, unused 4.3.4
    SPELL_AURA_MOD_IGNORE_SHAPESHIFT                        = 275,
    SPELL_AURA_MOD_DAMAGE_DONE_FOR_MECHANIC                 = 276,
    SPELL_AURA_277                                          = 277, // old SPELL_AURA_MOD_MAX_AFFECTED_TARGETS. unused 4.3.4
    SPELL_AURA_MOD_DISARM_RANGED                            = 278,
    SPELL_AURA_INITIALIZE_IMAGES                            = 279,
    SPELL_AURA_MOD_ARMOR_PENETRATION_PCT                    = 280, // old SPELL_AURA_MOD_ARMOR_PENETRATION_PCT unused 4.3.4
    SPELL_AURA_MOD_REPUTATION_GAIN_PCT                      = 281,
    SPELL_AURA_282                                          = 282,
    SPELL_AURA_MOD_HEALING_RECEIVED                         = 283,  // Possibly only for some spell family class spells
    SPELL_AURA_LINKED                                       = 284,
    SPELL_AURA_285                                          = 285,
    SPELL_AURA_286                                          = 286,
    SPELL_AURA_DEFLECT_SPELLS                               = 287,
    SPELL_AURA_IGNORE_HIT_DIRECTION                         = 288,
    SPELL_AURA_PREVENT_DURABILITY_LOSS                      = 289,
    SPELL_AURA_MOD_CRIT_PCT                                 = 290,
    SPELL_AURA_MOD_XP_QUEST_PCT                             = 291,
    SPELL_AURA_292                                          = 292,
    SPELL_AURA_OVERRIDE_SPELLS                              = 293,
    SPELL_AURA_PREVENT_REGENERATE_POWER                     = 294,
    SPELL_AURA_295                                          = 295, // unused
    SPELL_AURA_SET_VEHICLE_ID                               = 296,
    SPELL_AURA_BLOCK_SPELL_FAMILY                           = 297,
    SPELL_AURA_STRANGULATE                                  = 298,
    SPELL_AURA_299                                          = 299, // unused
    SPELL_AURA_SHARE_DAMAGE_PCT                             = 300,
    SPELL_AURA_SCHOOL_HEAL_ABSORB                           = 301,
    SPELL_AURA_302                                          = 302, // unused
    SPELL_AURA_MOD_DAMAGE_DONE_VERSUS_AURASTATE             = 303,
    SPELL_AURA_MOD_FAKE_INEBRIATE                           = 304,
    SPELL_AURA_MOD_MINIMUM_SPEED                            = 305,
    SPELL_AURA_306                                          = 306, // unused
    SPELL_AURA_HEAL_ABSORB_TEST                             = 307,
    SPELL_AURA_MOD_CRIT_CHANCE_FOR_CASTER                   = 308,
    SPELL_AURA_MOD_RESILIENCE_PCT                           = 309,
    SPELL_AURA_MOD_CREATURE_AOE_DAMAGE_AVOIDANCE            = 310,
    SPELL_AURA_311                                          = 311, // unk
    SPELL_AURA_ANIM_REPLACEMENT_SET                         = 312,
    SPELL_AURA_313                                          = 313,  // Not used in 4.3.4 - related to mounts
    SPELL_AURA_PREVENT_RESURRECTION                         = 314,
    SPELL_AURA_UNDERWATER_WALKING                           = 315,
    SPELL_AURA_316                                          = 316,  // Not used in 4.3.4 (name from 3.3.5a)
    SPELL_AURA_MOD_SPELL_POWER_PCT                          = 317,
    SPELL_AURA_MASTERY                                      = 318,
    SPELL_AURA_MOD_MELEE_ATTACK_SPEED                       = 319,
    SPELL_AURA_MOD_RANGED_HASTE_3                           = 320,
    SPELL_AURA_MOD_NO_ACTIONS                               = 321,
    SPELL_AURA_INTERFERE_TARGETTING                         = 322,
    SPELL_AURA_MOD_CRITICAL_DAMAGE_FROM_MEELE_SPELLS        = 323,
    SPELL_AURA_324                                          = 324,  // spell critical chance (probably by school mask)
    SPELL_AURA_325                                          = 325,  // Not used in 4.3.4
    SPELL_AURA_PHASE_2                                      = 326,  // phase related
    SPELL_AURA_FORCE_TO_SEE_ALL_PHASES                      = 327,
    SPELL_AURA_PROC_ON_POWER_PCT                            = 328,
    SPELL_AURA_MOD_POWER_GENERATE_PCT                       = 329,
    SPELL_AURA_CAST_WHILE_WALKING                           = 330,
    SPELL_AURA_FORCE_WEATHER                                = 331,
    SPELL_AURA_OVERRIDE_ACTIONBAR_SPELLS                    = 332,
    SPELL_AURA_OVERRIDE_ACTIONBAR_SPELLS_2                  = 333,
    SPELL_AURA_MOD_BLIND                                    = 334,  // NYI
    SPELL_AURA_SEE_WHILE_INVISIBLE                          = 335,
    SPELL_AURA_MOD_FLYING_RESTRICTIONS                      = 336,  // NYI
    SPELL_AURA_MOD_VENDOR_ITEMS_PRICES                      = 337,
    SPELL_AURA_MOD_DURABILITY_LOSS                          = 338,
    SPELL_AURA_INCREASE_SKILL_GAIN_CHANCE                   = 339,  // NYI
    SPELL_AURA_MOD_RESURRECTED_HEALTH_BY_GUILD_MEMBER       = 340,  // Increases health gained when resurrected by a guild member by X
    SPELL_AURA_MOD_SPELL_CATEGORY_COOLDOWN                  = 341,
    SPELL_AURA_MOD_MELEE_AND_RANGED_ATTACK_SPEED_PCT        = 342,
    SPELL_AURA_MOD_AUTOATTACK_DAMAGE_TARGET                 = 343,  // unk
    SPELL_AURA_MOD_AUTOATTACK_DAMAGE                        = 344,  // NYI
    SPELL_AURA_BYPASS_ARMOR_FOR_CASTER                      = 345,
    SPELL_AURA_ENABLE_ALT_POWER                             = 346,  // NYI
    SPELL_AURA_MOD_SPELL_COOLDOWN_BY_HASTE                  = 347,  // NYI
    SPELL_AURA_348                                          = 348,
    SPELL_AURA_MOD_CURRENCY_GAIN                            = 349,
    SPELL_AURA_350                                          = 350,
    SPELL_AURA_MOD_CURRENCY_LOOT                            = 351,
    SPELL_AURA_ALLOW_WORGEN_TRANSFORM                       = 352,
    SPELL_AURA_353                                          = 353,
    SPELL_AURA_MOD_HEALING_DONE_FROM_PCT_HEALTH             = 354,  // Restoration Shaman mastery - mod healing based on target's health (less = more healing)
    SPELL_AURA_MOD_CASTING_SPEED                            = 355,
    SPELL_AURA_356                                          = 356,
    SPELL_AURA_ENABLE_BOSS1_UNIT_FRAME                      = 357,
    SPELL_AURA_WORGEN_ALTERED_FORM                          = 358,
    SPELL_AURA_MOD_HEALING_DONE2                            = 359,
    SPELL_AURA_PROC_TRIGGER_SPELL_COPY                      = 360,   // Procs the same spell that caused this proc (Dragonwrath, Tarecgosa's Rest)
    SPELL_AURA_PROC_MELEE_TRIGGER_SPELL                     = 361,
    SPELL_AURA_DUMMY2                                       = 362,  // used in 143400
    SPELL_AURA_MOD_NEXT_SPELL                               = 363,  // Used by 101601 Throw Totem - causes the client to initialize spell cast with specified spell
    SPELL_AURA_364                                          = 364,  // Not used in 4.3.4
    SPELL_AURA_MAX_FAR_CLIP_PLANE                           = 365,  // Overrides client's View Distance setting to max("Fair", current_setting) and turns off terrain display
    SPELL_AURA_OVERRIDE_SPELL_POWER_BY_AP_PCT               = 366,  // Sets spellpower equal to % of attack power, discarding all other bonuses (from gear and buffs)
    SPELL_AURA_OVERRIDE_AUTOATTACK                          = 367,
    SPELL_AURA_MOD_DISARM2                                  = 368,  // May be two weapon affected?
    SPELL_AURA_ENABLE_POWER_BAR_TIMER                       = 369,
    SPELL_AURA_370                                          = 370,
    SPELL_AURA_DISABLE_ATTACK_AND_CAST                      = 371,
    SPELL_AURA_MOUNTED_VISUAL                               = 372,
    SPELL_AURA_MOD_SPEED_NO_CONTROL                         = 373,
    SPELL_AURA_MOD_FALL_DAMAGE                              = 374,
    SPELL_AURA_375                                          = 375,  // 0 auras
    SPELL_AURA_376                                          = 376,
    SPELL_AURA_CAST_WHILE_WALKING2                          = 377,
    SPELL_AURA_378                                          = 378,  // 0 auras
    SPELL_AURA_MOD_MANA_REGEN_PERCENT                       = 379,
    SPELL_AURA_MOD_GLOBAL_COOLDOWN_BY_HASTE                 = 380,
    SPELL_AURA_381                                          = 381,
    SPELL_AURA_MOD_PET_STATS_MODIFIER                       = 382,
    SPELL_AURA_IGNORE_CD                                    = 383,  // aura to ignore some spells cd? (Incarnation, Ascendance, Relentless Grip)
    SPELL_AURA_384                                          = 384,  // 0 auras
    SPELL_AURA_385                                          = 385,
    SPELL_AURA_386                                          = 386,
    SPELL_AURA_387                                          = 387,
    SPELL_AURA_MOD_FLY_PATH_SPEED                           = 388,
    SPELL_AURA_389                                          = 389,
    SPELL_AURA_390                                          = 390,  // 0 auras
    SPELL_AURA_391                                          = 391,  // 0 auras
    SPELL_AURA_392                                          = 392,  // 0 auras
    SPELL_AURA_MOD_DEFLECT_SPELLS_FROM_FRONT                = 393,
    SPELL_AURA_SHOW_CONFIRMATION_PROMPT                     = 394,
    SPELL_AURA_CREATE_AREATRIGGER                           = 395,  // create areatrigger
    SPELL_AURA_PROC_ON_POWER_AMOUNT                         = 396,
    SPELL_AURA_BATTLEGROUND_FLAG                            = 397,
    SPELL_AURA_BATTLEGROUND_FLAG_2                          = 398,
    SPELL_AURA_MOD_TIME_RATE                                = 399,
    SPELL_AURA_MOD_SKILL_2                                  = 400,
    SPELL_AURA_CART_AURA                                    = 401,  // cart capture points
    SPELL_AURA_ENABLE_POWER_TYPE                            = 402,
    SPELL_AURA_OVERRIDE_SPELL_VISUAL                        = 403,
    SPELL_AURA_OVERRIDE_AP_BY_SPELL_POWER_PCT               = 404,  // Sets attack power equal to % of spellpower, discarding all other bonuses (from gear and buffs)
    SPELL_AURA_INCREASE_RAITING_BY_PCT                      = 405,  // SPELL_AURA_INCREASE_STATS_FROM_ITEMS_BY_PCT
    SPELL_AURA_OVERRIDE_CLIENT_CONTROLS                     = 406,
    SPELL_AURA_MOD_FEAR_2                                   = 407,
    SPELL_AURA_PROC_SPELL_CHARGE                            = 408,
    SPELL_AURA_CAN_TURN_WHILE_FALLING                       = 409,
    SPELL_AURA_MOD_EXTRA_PET_DAMAGE                         = 410,  // Glyph of Stampede
    SPELL_AURA_MOD_MAX_CHARGES                              = 411,
    SPELL_AURA_HASTE_AFFECTS_BASE_MANA_REGEN                = 412,
    SPELL_AURA_MOD_PARRY_PERCENT2                           = 413,  // 3 auras
    SPELL_AURA_MOD_DEFLECT_RANGED_ATTACKS                   = 414,
    SPELL_AURA_415                                          = 415,  // 5.0.5 unk aura
    SPELL_AURA_MOD_COOLDOWN_BY_HASTE_REGEN                  = 416,
    SPELL_AURA_MOD_GLOBAL_COOLDOWN_BY_HASTE_REGEN           = 417,
    SPELL_AURA_MOD_MAX_POWER                                = 418,
    SPELL_AURA_MOD_ADD_ENERGY_PERCENT                       = 419,
    SPELL_AURA_MOD_BATTLE_PET_XP_GAIN                       = 420,
    SPELL_AURA_MOD_ABSORB_AMOUNT                            = 421,
    SPELL_AURA_MOD_ABSORBTION_PERCENT                       = 422,
    SPELL_AURA_423                                          = 423,  // Heart of the Wild (Talent)
    SPELL_AURA_424                                          = 424,  // Heart of the Wild (Talent)
    SPELL_AURA_425                                          = 425,  // 0 spells
    SPELL_AURA_426                                          = 426,  // Ring of Valor - In the Arena (Fight)
    SPELL_AURA_ITEMS_SCALING_AURA                           = 427,
    SPELL_AURA_SUMMON_CONTROLLER                            = 428,
    SPELL_AURA_PET_DAMAGE_DONE_PCT                          = 429,
    SPELL_AURA_ACTIVATE_SCENE                               = 430,
    SPELL_AURA_CONTESTED_PVP                                = 431,
    SPELL_AURA_432                                          = 432,
    SPELL_AURA_433                                          = 433,
    SPELL_AURA_434                                          = 434,
    SPELL_AURA_435                                          = 435,  // 0 spells
    SPELL_AURA_MOD_ENVIRONMENTAL_DAMAGE_TAKEN               = 436,  // 1 test spell
    SPELL_AURA_MOD_MINIMUM_SPEED_RATE                       = 437,
    SPELL_AURA_PRELOAD_PHASE                                = 438,
    SPELL_AURA_439                                          = 439,
    SPELL_AURA_440                                          = 440,
    SPELL_AURA_441                                          = 441,
    SPELL_AURA_442                                          = 442,
    SPELL_AURA_MOD_LIFE_STEAL_PCT                           = 443,
    SPELL_AURA_444                                          = 444,
    SPELL_AURA_445                                          = 445,
    SPELL_AURA_446                                          = 446,
    SPELL_AURA_MOD_XP_FROM_CREATURE_TYPE                    = 447,
    SPELL_AURA_MOD_CRITICAL_HEAl_PCT                        = 448,
    SPELL_AURA_449                                          = 449,
    SPELL_AURA_UPGRADE_CHARACTER_SPELL_TIER                 = 450,
    SPELL_AURA_OVERRIDE_PET_SPECS                           = 451, // NYI
    SPELL_AURA_452                                          = 452,
    SPELL_AURA_CHARGE_RECOVERY_MOD                          = 453,
    SPELL_AURA_CHARGE_RECOVERY_MULTIPLIER                   = 454,
    SPELL_AURA_MOD_ROOTED                                   = 455,
    SPELL_AURA_CHARGE_RECOVERY_AFFECTED_BY_HASTE            = 456,
    SPELL_AURA_CHARGE_RECOVERY_AFFECTED_BY_HASTE_REGEN      = 457,
    SPELL_AURA_IGNORE_DUAL_WIELD_HIT_PENALTY                = 458,
    SPELL_AURA_DISABLE_MOVEMENT_FORCE                       = 459,
    SPELL_AURA_RESET_COOLDOWNS_AT_DUEL_START                = 460,
    SPELL_AURA_461                                          = 461,
    SPELL_AURA_MOD_HEALING_AND_ABSORB_FROM_CASTER           = 462,
    SPELL_AURA_CONVERT_CRIT_RATING_PCT_TO_PARRY_RATING      = 463,
    SPELL_AURA_464                                          = 464,
    SPELL_AURA_MOD_BONUS_ARMOR                              = 465,
    SPELL_AURA_MOD_BONUS_ARMOR_PCT                          = 466, // Affects bonus armor gain from all sources except base stats
    SPELL_AURA_MOD_STAT_BONUS_PCT                           = 467, // Affects stat gain from all sources except base stats
    SPELL_AURA_PROC_ON_HP_BELOW                             = 468,
    SPELL_AURA_SHOW_CONFIRMATION_PROMPT_WITH_DIFFICULTY     = 469,
    SPELL_AURA_EXPEDITE                                     = 470,
    SPELL_AURA_MOD_VERSALITY_PCT                            = 471,
    SPELL_AURA_FIXATE2                                      = 472,
    SPELL_AURA_PREVENT_DURABILITY_LOSS_FROM_COMBAT          = 473,
    SPELL_AURA_MOD_DROP_ITEM_BONUSES_CHANCE                 = 474,
    SPELL_AURA_ALLOW_USING_GAMEOBJECTS_WHILE_MOUNTED        = 475,
    SPELL_AURA_MOD_CURRENCY_GAIN_LOOTED                     = 476,
    SPELL_AURA_DRAENOR_SPELLS_AND_ITEMS_SCALING             = 477,
    SPELL_AURA_478                                          = 478,
    SPELL_AURA_479                                          = 479,
    SPELL_AURA_480                                          = 480,
    SPELL_AURA_CONVERT_CONSUMED_RUNE                        = 481,
    SPELL_AURA_CAMERA_SELFIE                                = 482,
    SPELL_AURA_TRANSFORM_SUPRESSION_IN_PVP                  = 483,
    SPELL_AURA_ALLOW_KICK_SPECIFIC_SPELL_WITH_ID            = 484,
    SPELL_AURA_MOD_GRAVITY_SPEED                            = 485,
    SPELL_AURA_INTERFERE_TARGETTING_2                       = 486,
    SPELL_AURA_487                                          = 487,
    SPELL_AURA_DISABLE_GRAVITY                              = 488, // not sure
    SPELL_AURA_FORGET_LANGUAGE                              = 489,
    SPELL_AURA_SWITCH_TEAM                                  = 490,
    SPELL_AURA_SPELL_AURA_MOD_HONOR_POINTS_GAIN             = 491,
    SPELL_AURA_492                                          = 492,

    TOTAL_AURAS
};

enum AuraObjectType
{
    UNIT_AURA_TYPE,
    DYNOBJ_AURA_TYPE,
    AREA_TRIGGER_AURA_TYPE,
};
#endif
